from django.db.models import Q
from greg.models import Identity, Person

def person_by_string_query(request):
    search = request.query_params["q"]
    if "gregui" in request.version:
        id_field_name = "pid"
    else:
        id_field_name = "person_id"

    split_search = search.split()
    words_joined = "|".join(map(str, split_search))
    # Create a regex with the terms in the search or-ed together. This will trigger a match
    # if one of the fields that are being searched contains one of the terms
    search_regex = r"^(%s)" % words_joined

    hits = []
    # First look for hits on name and birth date
    persons = Person.objects.filter(
        Q(first_name__iregex=search_regex)
        | Q(last_name__iregex=search_regex)
        | Q(date_of_birth__iregex=search_regex)
    )[:10]

    included_persons = []
    for person in persons:
        hits.append(
            {
                id_field_name: person.id,
                "first": person.first_name,
                "last": person.last_name,
                "date_of_birth": person.date_of_birth,
            }
        )
        included_persons.append(person.id)

    if len(hits) == 10:
        # Max number of hits, no need to search more
        return hits

    # Look for hits in e-mail and mobile phone
    identities = Identity.objects.filter(
        value__iregex=search_regex,
        type__in=[
            Identity.IdentityType.PRIVATE_EMAIL,
            Identity.IdentityType.PRIVATE_MOBILE_NUMBER,
        ],
    )[: (10 - len(hits))]

    for identity in identities:
        if identity.person_id in included_persons:
            continue

        hits.append(
            {
                id_field_name: identity.person_id,
                "first": identity.person.first_name,
                "last": identity.person.last_name,
                "value": identity.value,
                "type": identity.type,
            }
        )

    return hits
